#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include "kbs_Error.h"
#include "kbs_String.h"
#include "kbs_Types.h"



/* ------------------------------------------------------------------------*/
Kbs_Ustring* kbs_convert_Ustring2RangeExtend(Kbs_Ustring *const ustring, const Kbs_Ulong sizeExt) {
	if (ustring == NULL) {
		KBS_ERROR(KBS_ERROR_NULLPOINTER);
		return NULL;
	}
	if (ustring->alphabet == NULL) {
		kbs_get_AlphabetForUstring(ustring);
	}
	Kbs_Ulong strLen = ustring->strLength;
	ustring->str = (Kbs_Uchar *)realloc(ustring->str, (strLen + sizeExt) * sizeof(Kbs_Uchar));
	if (ustring->str == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	Kbs_Uchar *charPtr = ustring->str;
	Kbs_Ulong i;
	Kbs_Int *alphaMap = ustring->alphabet->alphaMapping;
	for (i=0; i<strLen; i++) {
		*charPtr = alphaMap[*charPtr];
		charPtr++;
	}
	memset(charPtr, '\0', sizeof(Kbs_Uchar)*sizeExt);
	Kbs_Int alphabetSize = ustring->alphabet->alphaSize;
	Kbs_Uchar *charArray = ustring->alphabet->charArray;
	Kbs_Int j;
	Kbs_Ulong *charFreq = ustring->alphabet->charFreq;
	for(j=0; j<alphabetSize; j++) {
		charFreq[j] = charFreq[charArray[j]];
		charArray[j] = (Kbs_Uchar) j;
		alphaMap[j] = j;
	}
	for(; j<KBS_MAX_ALPHABET_SIZE; j++) {
		alphaMap[j] = (Kbs_Int) (-1);
	}
	return ustring;
}




/* ------------------------------------------------------------------------*/
Kbs_Ustring* kbs_getUstring_FromFile(const Kbs_Char *const filename) {
	struct stat fileInfo;
	Kbs_Ustring *newStr;
	FILE *file;
	if(filename == NULL) {
		KBS_ERROR(KBS_ERROR_NULLPOINTER);
		return NULL;
	}
	newStr = (Kbs_Ustring *) malloc(sizeof(Kbs_Ustring));
	if (newStr == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	if ( stat(filename,	&fileInfo ) ) {
		free(newStr);
	  printf("Unable to get stat of file %s \n", filename);
		KBS_ERROR(KBS_ERROR_FILEOPEN);
		return NULL;
	}
	newStr->strLength = fileInfo.st_size;
	newStr->str = (Kbs_Uchar *) malloc(((fileInfo.st_size) + 1) * sizeof(Kbs_Uchar));
	if (newStr->str == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	if ( !(file = fopen(filename, "r")) ) {
		free(newStr->str);
		free(newStr);
		printf("Unable to open file %s \n", filename);
		KBS_ERROR(KBS_ERROR_FILEOPEN);
		return NULL;
	}
	rewind(file);
	if ( newStr->strLength > fread(newStr->str, sizeof(Kbs_Uchar), newStr->strLength, file) ) {
		fclose(file);
		free(newStr->str);
		free(newStr);
		KBS_ERROR(KBS_ERROR_FILEOPEN);
		return NULL;
	}
	fclose(file);
	newStr->str[newStr->strLength] = '\0';
	return newStr;
}



/*------------------------------------------------------------------------*/
Kbs_Ustring* kbs_getUstringWithAlphabet_FromFile(Kbs_Char *filename) {
	Kbs_Ustring *newStr;
	newStr = kbs_getUstring_FromFile(filename);
	kbs_get_AlphabetForUstring(newStr);
	return newStr;
}


/*------------------------------------------------------------------------*/
void kbs_delete_Ustring(Kbs_Ustring* oldStr) {
	if(oldStr == NULL) {
		return;
	}
	if (oldStr->str != NULL){
		free(oldStr->str );
	}
	kbs_delete_Alphabet(oldStr->alphabet);
	free(oldStr);
}



/*----------------------------------------------------------------------------*/
void kbs_get_AlphabetForUstring(Kbs_Ustring *thisString) {
	Kbs_Ulong i;
	Kbs_Alphabet* alphabet;
	Kbs_Ulong stringLength;
	Kbs_Uchar tmpChar;
	if (thisString == NULL) {
		fprintf(stdout, "File %s line %u: Try to get alphabet of Null-String\n", __FILE__, __LINE__ );
		return;
	}
	alphabet = (Kbs_Alphabet *) malloc(sizeof(Kbs_Alphabet));
	if (alphabet == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	alphabet->charArray = (Kbs_Uchar *) calloc(KBS_MAX_ALPHABET_SIZE+1, sizeof(Kbs_Uchar));
	if (alphabet->charArray == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	for (i=0; i<KBS_MAX_ALPHABET_SIZE; i++) {
		alphabet->alphaMapping[i] = -1;
		alphabet->charFreq[i] = 0;
	}
	alphabet->alphaSize = 0;
	stringLength = thisString->strLength;
	for (i=0; i<stringLength; i++) {
		tmpChar = thisString->str[i];
		if (alphabet->alphaMapping[tmpChar] < 0) {
			alphabet->alphaMapping[tmpChar] = alphabet->alphaSize;
			alphabet->charArray[alphabet->alphaSize] = tmpChar;
			alphabet->alphaSize++;
		}
		alphabet->charFreq[tmpChar] = alphabet->charFreq[tmpChar] + 1;
	}
	alphabet->charArray = (Kbs_Uchar *)realloc(alphabet->charArray, (alphabet->alphaSize + 1) * sizeof(Kbs_Uchar));
	if (alphabet->charArray == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	kbs_sort_Alphabet(alphabet);
	thisString->alphabet = alphabet;
}



