#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "kbs_Alphabet.h"
#include "kbs_Error.h"
#include "kbs_Limits.h"
#include "kbs_Math.h"
#include "kbs_String.h"
#include "kbs_SuffixArray.h"
#include "kbs_SuffixArrayConstDStepAndPre.h"
#include "kbs_Time.h"
#include "kbs_Types.h"



__inline__ static void insSortPtrLevel_SaBucket(register Kbs_Ulong *const leftPtr, Kbs_Ulong *const rightPtr, register const Kbs_Ulong offset, register const Kbs_Ulong **const sufPtrMap);
__inline__ static Kbs_Ulong medianOfThreeUlong(const Kbs_Ulong *const a, const Kbs_Ulong *const b, const Kbs_Ulong *const c);
static void qSortInsSortPtrLevel_SaBucket(Kbs_Ulong *const leftPtr, register Kbs_Ulong *const rightPtr, register const Kbs_Ulong offset, const Kbs_Ulong **const sufPtrMap);

__inline__ static void computeBucketSize2_SaBucket(register Kbs_Ulong *const leftPtr, register Kbs_Ulong *const rightPtr, register const Kbs_Ulong d, register const Kbs_Ulong offset, register Kbs_Ulong **const sufPtrMap);
__inline__ static Kbs_Ulong computeDiffDepthBucket_SaBucket(register Kbs_Ulong *const leftPtr, register Kbs_Ulong *const rightPtr, register const Kbs_Ulong d, register const Kbs_Ulong offset, register Kbs_Ulong **const sufPtrMap);

static void updatePtrAndRefineBuckets_SaBucket(Kbs_Ulong *leftPtr, Kbs_Ulong *rightPtr, const Kbs_Ulong d, const Kbs_Ulong offset, Kbs_Ulong **sufPtrMap);

static void insSortViaPtr_SaBucket(Kbs_Ulong *leftPtr, Kbs_Ulong *rightPtr, const Kbs_Ulong d, const Kbs_Ulong offset, Kbs_Ulong **sufPtrMap);
static void qSortInsSortViaPtr_SaBucket(Kbs_Ulong *leftPtr, Kbs_Ulong *rightPtr, const Kbs_Ulong d, const Kbs_Ulong offset,Kbs_Ulong **sufPtrMap);

static Kbs_Ulong **determineAll_Buckets_Sarray(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr);
static Kbs_Ulong **determinePower2Alpha_Buckets_Sarray(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr);
static Kbs_Ulong **determine_Buckets_Sarray(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr);

static Kbs_Ulong **determineAll_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr, Kbs_Ulong ***sufPtrMapPtr);
static Kbs_Ulong **determinePower2Alpha_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr, Kbs_Ulong ***sufPtrMapPtr);
static Kbs_Ulong **determine_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr, Kbs_Ulong ***sufPtrMapPtr);

static Kbs_Uchar *getCharWeightedOrder_Alphabet(const Kbs_Alphabet *const alphabet, const Kbs_Ulong **const buckets, const Kbs_Ulong bucketsInLevel2Bucket, const Kbs_Ulong weightOnCC);




/**
 * Determines the maximal q-gram size d for the given alphabet size and
 * @param alphabetSize
 * @return maximal q-gram size
 */
__inline__ static Kbs_Ulong determineMaxD(const Kbs_Ulong alphabetSize) {
	if (alphabetSize == 1) {
		return 10000;
	}
	Kbs_Ulong numberHashValues = 1;
	Kbs_Ulong d = 0;
	const Kbs_Ulong bound = KBS_ULONG_MAX / (alphabetSize * sizeof(Kbs_Ulong));
	while (numberHashValues <= bound){
		numberHashValues *= alphabetSize;
		d++;
	}
	return d;
}

/*---------------------------------------------------------------------------------------*/
/**
 * Performs the stable sort of one level of a bucket concerning the pointers in the sufPtrMap
 * @param leftPtr points to the leftmost position of the current bucket
 * @param rightPtr points to the rightmost position of the current bucket
 * @param offset denotes that contained suffixes share same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
 */
__inline__ static void insSortPtrLevel_SaBucket(register Kbs_Ulong *const leftPtr,
																			Kbs_Ulong *const rightPtr,
																			register const Kbs_Ulong offset,
																			register const Kbs_Ulong **const sufPtrMap) {

	register Kbs_Ulong *rightTmpPtr = leftPtr+1;
	while(rightTmpPtr <= rightPtr) {
		register const Kbs_Ulong tempValue = *rightTmpPtr;
		register const Kbs_Ulong *const tempHashValue = sufPtrMap[(*rightTmpPtr)+offset];
		register Kbs_Ulong *leftTmpPtr = rightTmpPtr;
		while(leftTmpPtr > leftPtr && sufPtrMap[*(leftTmpPtr-1)+offset] > tempHashValue) {
			*leftTmpPtr = *(leftTmpPtr-1);
			leftTmpPtr--;
		}
		*leftTmpPtr = tempValue;
		rightTmpPtr++;
	}
}



/*---------------------------------------------------------------------------------------*/
/**
 * @param a first key
 * @param b second key
 * @param c third key
 * @return median of a,b,c
 */
__inline__ static Kbs_Ulong medianOfThreeUlong(const Kbs_Ulong *const a, const Kbs_Ulong *const b, const Kbs_Ulong *const c) {
	if (a==b) {
		return 0;
	}
	if (a==c || b==c) {
		return 2;
	}
	return a<b ?(b<c ? 1 : (a<c ? 2 : 0)) : (b>c ? 1 : (a<c ? 0 : 2));
}




/*---------------------------------------------------------------------------------------*/
/**
 * Mixture of Quicksort and Insertion Sort
 * Partitions smaller than a given threshold are sorted by insertion sort
 * @param leftPtr points to the leftmost position of the current bucket
 * @param rightPtr points to the rightmost position of the current bucket
 * @param offset denotes that contained suffixes share same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
 */
static void qSortInsSortPtrLevel_SaBucket(Kbs_Ulong *const leftPtr,
																		register Kbs_Ulong *const rightPtr,
																		register const Kbs_Ulong offset,
																		const Kbs_Ulong **const sufPtrMap) {

	register const Kbs_Ulong halfBucketSize = (Kbs_Ulong)(rightPtr - leftPtr)/2;
	register Kbs_Ulong *pivot = sufPtrMap[*rightPtr + offset];
	register Kbs_Ulong *const pivotb = sufPtrMap[*leftPtr + offset];
	register Kbs_Ulong *const pivotc = sufPtrMap[*(leftPtr+halfBucketSize) + offset];
	register const Kbs_Int medNumber = medianOfThreeUlong(pivot, pivotb, pivotc);
	if (medNumber > 0){
		register Kbs_Ulong *pivotPtr = (medNumber == 1) ? leftPtr : (leftPtr+halfBucketSize);
		pivot = (medNumber == 1) ? pivotb : pivotc;
		register const Kbs_Ulong swapTmp = *pivotPtr;
		*pivotPtr = *rightPtr;
		*rightPtr = swapTmp;
	}
	register Kbs_Ulong *leftTmpPtr = leftPtr;
	register Kbs_Ulong *rightTmpPtr = leftPtr;
	while (rightTmpPtr < rightPtr) {
		if (sufPtrMap[*rightTmpPtr + offset] <= pivot) {
			register const Kbs_Ulong swapTmp = *leftTmpPtr;
			*leftTmpPtr = *rightTmpPtr;
			*rightTmpPtr = swapTmp;
			leftTmpPtr++;
		}
		rightTmpPtr++;
	}
	register const Kbs_Ulong swapTmp = *leftTmpPtr;
	*leftTmpPtr = *rightTmpPtr;
	*rightTmpPtr = swapTmp;
	register Kbs_Ulong partSize;
	if ((partSize = rightPtr-leftTmpPtr) > 1) {
 		if (partSize < KBS_INSSORT_THRES_LEN) { /* sort by insertion sort */
 			insSortPtrLevel_SaBucket(leftTmpPtr+1, rightPtr, offset, sufPtrMap);
 		}
 		else {
			qSortInsSortPtrLevel_SaBucket(leftTmpPtr+1, rightPtr, offset, sufPtrMap);
		}
	}
	do { 									/* extends the pivot region, as long as, it equals the pivot */
		leftTmpPtr--;
	} while (leftTmpPtr > leftPtr && sufPtrMap[*leftTmpPtr+offset] == pivot);
	if ((partSize = leftTmpPtr-leftPtr) > 0) {
 		if (partSize < KBS_INSSORT_THRES_LEN) { /* sort by insertion sort */
 			insSortPtrLevel_SaBucket(leftPtr, leftTmpPtr, offset, sufPtrMap);
 		}
 		else {
			qSortInsSortPtrLevel_SaBucket(leftPtr, leftTmpPtr, offset, sufPtrMap);
 		}
	}
}



/*---------------------------------------------------------------------------------------*/
/**
 * Completely sort buckets of size 2
 * @param leftPtr points to the left suffix of the current bucket
 * @param rightPtr points to the right suffix of the current bucket
 * @param d is the initial prefix length used for the bucket sort. It also determines the increase of offset
 * @param offset denotes that contained suffixes share same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
 */
__inline__ static void computeBucketSize2_SaBucket(register Kbs_Ulong *const leftPtr,
																		register Kbs_Ulong *const rightPtr,
																		register const Kbs_Ulong d,
																		register const Kbs_Ulong offset,
																		register Kbs_Ulong **const sufPtrMap) {
	register Kbs_Ulong suffix1 = *leftPtr + offset;
	register Kbs_Ulong suffix2 = *rightPtr + offset;
	while (sufPtrMap[suffix1] == sufPtrMap[suffix2]) {
		suffix1 += d;
		suffix2 += d;
	}
	if (sufPtrMap[suffix1] > sufPtrMap[suffix2]) {
		register const Kbs_Ulong tmpSwap = *leftPtr;
		*leftPtr = *rightPtr;
		*rightPtr = tmpSwap;
	}
	sufPtrMap[*leftPtr] = leftPtr;
}


/*---------------------------------------------------------------------------------------*/
/**
 * Computes the LCP of all suffixes in this bucket. This LCP will be the newoffset
 * @param leftPtr points to the left suffix of the current bucket
 * @param rightPtr points to the right suffix of the current bucket
 * @param d is the initial prefix length used for the bucket sort. It also determines the increase of offset
 * @param offset denotes that contained suffixes share same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
 * @return the LCP of suffixes in this bucket
 */
__inline__ static Kbs_Ulong computeDiffDepthBucket_SaBucket(register Kbs_Ulong *const leftPtr,
																		register Kbs_Ulong *const rightPtr,
																		register const Kbs_Ulong d,
																		register const Kbs_Ulong offset,
																		register Kbs_Ulong **const sufPtrMap) {
	register Kbs_Ulong lcp = offset;
	while (1) {
		register Kbs_Ulong *runPtr = leftPtr;
		register Kbs_Ulong *const tmpPtr = sufPtrMap[*rightPtr + lcp];
		while(runPtr < rightPtr) {
			if(sufPtrMap[*runPtr+lcp] != tmpPtr) {
				return lcp;
			}
			runPtr++;
		}
		lcp +=d;
	}
}





/*---------------------------------------------------------------------------------------*/
/**
 * The function determines the subbuckets of this bucket
 * and recursively calls the refinement function for the sorting of 2 levels in one step from left to right
 * @param leftPtr points to the leftmost suffix of the current bucket
 * @param rightPtr points to the rightmost suffix of the current bucket
 * @param d is the initial prefix length used for the bucket sort. It also determines the increase of offset
 * @param offset denotes that contained suffixes share the same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
 */
static void updatePtrAndRefineBuckets_SaBucket(register Kbs_Ulong *const leftPtr,
																		register Kbs_Ulong *const rightPtr,
																		const Kbs_Ulong d,
																		register const Kbs_Ulong offset,
																		register Kbs_Ulong **const sufPtrMap) {

	/* for all buckets with resp. pointer > rightPtr determine buckets via setting sufPtrMap */
	register Kbs_Ulong *leftIntervalPtr = rightPtr;
	register Kbs_Ulong *rightIntervalPtr = rightPtr;
	register Kbs_Ulong *tmpPtr;
	while(leftPtr <= leftIntervalPtr && rightPtr < (tmpPtr = sufPtrMap[*leftIntervalPtr + offset] )) {
		do{
			sufPtrMap[*leftIntervalPtr] = rightIntervalPtr;
			leftIntervalPtr--;
		} while (leftPtr <= leftIntervalPtr
							&& sufPtrMap[*leftIntervalPtr + offset] == tmpPtr);
		rightIntervalPtr = leftIntervalPtr;
	}

	/* since the sufPtrMap for the suffixes between leftPtr and rightPtr might change in previous 2 steps */
	/* determine the bucket concerning suffixptr+offset between leftPtr and rightPtr seperately */
	rightIntervalPtr = leftIntervalPtr;
	while (leftPtr <= leftIntervalPtr
					&& leftPtr <= sufPtrMap[*leftIntervalPtr + offset]
					&& sufPtrMap[*leftIntervalPtr+offset] <= rightPtr) {
		sufPtrMap[*leftIntervalPtr] = rightIntervalPtr;
		leftIntervalPtr--;
	}

	/* for all buckets with resp. pointer+offset < leftPtr determine buckets via setting sufPtrMap */
	/* start with rightIntervalPtr which indicates leftend-1 of bucket with resp. pointer+offset between*/
	/* leftPtr and rightPtr*/
	Kbs_Ulong *const middleRightPtr = rightIntervalPtr;
	Kbs_Ulong *const middleLeftPtr = leftIntervalPtr;
	rightIntervalPtr = leftIntervalPtr;
	while(leftPtr <= leftIntervalPtr) {
		register const Kbs_Ulong *tmpPtr2 = sufPtrMap[*leftIntervalPtr + offset];
		do{
			sufPtrMap[*leftIntervalPtr] = rightIntervalPtr;
			leftIntervalPtr--;
		} while (leftPtr <= leftIntervalPtr
							&& sufPtrMap[*leftIntervalPtr + offset] == tmpPtr2);
		rightIntervalPtr = leftIntervalPtr;
	}

	Kbs_Ulong newOffset = offset+d;
	if (sufPtrMap[*leftPtr]==rightPtr) {
		newOffset = computeDiffDepthBucket_SaBucket(leftPtr, rightPtr, d, newOffset, sufPtrMap);
	}
	register Kbs_Ulong *leftTmpPtr = leftPtr;
	while (leftTmpPtr < middleLeftPtr) {
		Kbs_Ulong *const rightTmpPtr = sufPtrMap[*leftTmpPtr];
		const Kbs_Ulong tmpLong = rightTmpPtr - leftTmpPtr;
		if (tmpLong > 0) {
			if (tmpLong == 1) {
				computeBucketSize2_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
				leftTmpPtr = rightTmpPtr + 1;
				continue;
			}
			if (tmpLong < KBS_INSSORT_THRES_LEN) { /* sort by insertion sort */
 				insSortViaPtr_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
 			}
 			else
				qSortInsSortViaPtr_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
 		}
		leftTmpPtr = rightTmpPtr + 1;
	}
	/* for buckets refering to this bucket, the offset can be doubled */
	if (middleRightPtr > middleLeftPtr+1) {
		if (middleRightPtr - middleLeftPtr == 2) {
			computeBucketSize2_SaBucket(middleLeftPtr+1, middleRightPtr, d, 2*offset, sufPtrMap);
		}
		else {
			if (middleRightPtr - middleLeftPtr < KBS_INSSORT_THRES_LEN) { /* sort by insertion sort */
 				insSortViaPtr_SaBucket(middleLeftPtr+1, middleRightPtr, d, 2*offset, sufPtrMap);
 			}
 			else {
				qSortInsSortViaPtr_SaBucket(middleLeftPtr+1, middleRightPtr, d, 2*offset, sufPtrMap);
			}
		}
	}
	leftTmpPtr = middleRightPtr + 1;
	while (leftTmpPtr < rightPtr) {
		Kbs_Ulong *const rightTmpPtr = sufPtrMap[*leftTmpPtr];
		const Kbs_Ulong tmpLong = rightTmpPtr - leftTmpPtr;
		if (tmpLong > 0) {
			if (tmpLong == 1) {
				computeBucketSize2_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
				leftTmpPtr = rightTmpPtr + 1;
				continue;
			}
			if (tmpLong < KBS_INSSORT_THRES_LEN) { /* sort by insertion sort */
 				insSortViaPtr_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
 			}
 			else
				qSortInsSortViaPtr_SaBucket(leftTmpPtr, rightTmpPtr, d, newOffset, sufPtrMap);
 		}
		leftTmpPtr = rightTmpPtr + 1;
	}
}



/**
 * Refine this bucket by using insertion sort internally
 * @param leftPtr points to the left suffix of the current bucket
 * @param rightPtr points to the right suffix of the current bucket
 * @param d is the initial prefix length used for the bucket sort. It also determines the increase of offset
 * @param offset denotes that contained suffixes share the same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
*/
static void insSortViaPtr_SaBucket(Kbs_Ulong *leftPtr,
																		Kbs_Ulong *rightPtr,
																		const Kbs_Ulong d,
																		const Kbs_Ulong offset,
																		Kbs_Ulong **sufPtrMap) {
	insSortPtrLevel_SaBucket(leftPtr, rightPtr, offset, sufPtrMap);
	updatePtrAndRefineBuckets_SaBucket(leftPtr, rightPtr, d, offset, sufPtrMap);
}


/**
 * Refine this bucket by using mixed Quick-Insertion-Sort internally
 * @param leftPtr points to the left suffix of the current bucket
 * @param rightPtr points to the right suffix of the current bucket
 * @param d is the initial prefix length used for the bucket sort. It also determines the increase of offset
 * @param offset denotes that contained suffixes share the same prefix of length offset, which is used to refer to the sort key
 * @param sufPtrMap at position i points to the rightmost position of the bucket that contains suffix i (bptr)
*/
static void qSortInsSortViaPtr_SaBucket(Kbs_Ulong *leftPtr,
																		Kbs_Ulong *rightPtr,
																		const Kbs_Ulong d,
																		const Kbs_Ulong offset,
																		Kbs_Ulong **sufPtrMap) {
	qSortInsSortPtrLevel_SaBucket(leftPtr, rightPtr, offset, sufPtrMap);
	updatePtrAndRefineBuckets_SaBucket(leftPtr, rightPtr, d, offset, sufPtrMap);
}


/**
 * Constructs all buckets w.r.t. q-gram size d and the up to prefix d sorted suffix array
 * Call determine_Buckets_Sarray(const Kbs_Ustring *const mappedString, register const Kbs_Ulong d, Kbs_Ulong **suffixArrayPtr)
 * instead
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @return buckets containing pointers into the suffix array
 * @see determine_Buckets_Sarray
 */
static Kbs_Ulong **determineAll_Buckets_Sarray(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr) {

	union bucket{
		Kbs_Ulong *bucketPtr;
		Kbs_Ulong bucketSize;
	};

	register const Kbs_Ulong alphabetSize = mappedString->alphabet->alphaSize;
	register const Kbs_Ulong numberBuckets = kbs_power_Ulong(alphabetSize, d);
	register union bucket *const buckets  = (union bucket *)malloc(sizeof(union bucket) * (numberBuckets+1));
	memset(buckets, 0, sizeof(union bucket) * (numberBuckets+1));

	/* computation of first hashvalue */
	register Kbs_Uchar *mappedUcharArray = mappedString->str;
	register Kbs_Ulong hashCode = 0;
	Kbs_Ulong tempPower = 1;
	register Kbs_Long i;
	for (i=d-1; i>=0; i--) {
		hashCode += mappedUcharArray[i] * tempPower;
		tempPower *= alphabetSize;
	}
	const Kbs_Ulong firstHashCode = hashCode;

	/* computation of the filling of buckets */
	register const Kbs_Ulong strLen = mappedString->strLength;
	register const Kbs_Ulong tempModulo = kbs_power_Ulong(alphabetSize, d-1);
	mappedUcharArray += d;
	buckets[hashCode].bucketSize++;
	register Kbs_Ulong j;
	for (j=1; j<strLen; j++) {
		hashCode = hashCode % tempModulo;
		hashCode *= alphabetSize;
		hashCode += *mappedUcharArray;
		mappedUcharArray++;
		buckets[hashCode].bucketSize++;
	}

	Kbs_Ulong *const suffixArray = (Kbs_Ulong *)malloc(sizeof(Kbs_Ulong) * strLen);
	if (suffixArray == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	/* computation of the bucket pointers, pointers into the suffix array */
	buckets[0].bucketPtr = suffixArray + buckets[0].bucketSize;
	for (j=1; j<=numberBuckets; j++){
		buckets[j].bucketPtr = buckets[j-1].bucketPtr + buckets[j].bucketSize;
	}

	/* computation of the suffix array */
	mappedUcharArray = mappedString->str + d;
	hashCode = firstHashCode ;
	for (j=0; j<strLen; j++) {
		buckets[hashCode].bucketPtr--;
		*(buckets[hashCode].bucketPtr) = j;
		hashCode = hashCode % tempModulo;
		hashCode *= alphabetSize;
		hashCode += *mappedUcharArray;
		mappedUcharArray++;
	}
	buckets[numberBuckets].bucketPtr = suffixArray + strLen;
	*suffixArrayPtr = suffixArray;
	return (Kbs_Ulong **)buckets;
}


/**
 * Constructs all buckets w.r.t. q-gram size d and the up to prefix d sorted suffix array
 * Pre.: mappedString->alphabet->alphaSize = 2^x for some x
 * Call determine_Buckets_Sarray
 * instead
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @return buckets containing pointers into the suffix array
 * @see determine_Buckets_Sarray
 */
static Kbs_Ulong **determinePower2Alpha_Buckets_Sarray(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr) {

	union bucket{
		Kbs_Ulong *bucketPtr;
		Kbs_Ulong bucketSize;
	};
	register const Kbs_Int exp2 = (Kbs_Int) kbs_getExp_Ulong(2, mappedString->alphabet->alphaSize);
	if (exp2 < 0) {
		KBS_ERROR(KBS_ERROR_VALUEOUTOFBOUNDS);
		return NULL;
	}
	/* initialization of bucket table */
	register const Kbs_Ulong numberBuckets = kbs_power_Ulong(mappedString->alphabet->alphaSize, d);
	register union bucket *const buckets = (union bucket *)malloc(sizeof(union bucket) * (numberBuckets+1));
	memset(buckets, '\0', sizeof(union bucket) * (numberBuckets+1));

	/* computation of first hashvalue */
	register Kbs_Uchar *mappedUcharArray = mappedString->str;
	register Kbs_Ulong j;
	register Kbs_Ulong hashCode = 0;
	for (j=0; j<d; j++) {
		hashCode = hashCode << exp2;
		hashCode += mappedUcharArray[j];
	}
	const Kbs_Ulong firstHashCode = hashCode;

	/* computation of the filling for the buckets */
	register Kbs_Ulong tempModulo = (Kbs_Ulong)0;
	tempModulo = ~tempModulo;
	tempModulo = tempModulo << (exp2*(d-1));
	tempModulo = ~tempModulo;
	register Kbs_Ulong strLen = mappedString->strLength;
	mappedUcharArray += d;
	buckets[hashCode].bucketSize++;
	for (j=1; j<strLen; j++) {
		hashCode = hashCode & tempModulo;
		hashCode = hashCode << exp2;
		hashCode = hashCode | (Kbs_Ulong)(*mappedUcharArray);
		mappedUcharArray++;
		buckets[hashCode].bucketSize++;
	}

	Kbs_Ulong *const suffixArray = (Kbs_Ulong *)malloc(sizeof(Kbs_Ulong) * strLen);
	if (suffixArray == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}
	/* computation of the bucket pointers, pointers into the suffix array */
	buckets[0].bucketPtr = suffixArray + buckets[0].bucketSize;
	for (j=1; j<=numberBuckets; j++){
		buckets[j].bucketPtr = buckets[j-1].bucketPtr + buckets[j].bucketSize;
	}

	/* computation of the suffix array */
	mappedUcharArray = mappedString->str + d;
	hashCode = firstHashCode ;
	for (j=0; j<strLen; j++) {
		buckets[hashCode].bucketPtr--;
		*(buckets[hashCode].bucketPtr) = j;
		hashCode = hashCode & tempModulo;
		hashCode = hashCode << exp2;
		hashCode = hashCode | (Kbs_Ulong)(*mappedUcharArray);
		mappedUcharArray++;
	}
	buckets[numberBuckets].bucketPtr = suffixArray+strLen;
	*suffixArrayPtr = suffixArray;
	return (Kbs_Ulong **)buckets;
}

/**
 * Constructs all buckets w.r.t. q-gram size d, and the up to prefix d sorted suffix array
 * Hint: This function has to be called
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @return buckets that contains ptr's into the suffix array
 */
static Kbs_Ulong **determine_Buckets_Sarray(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr) {

	if (kbs_getExp_Ulong(2, mappedString->alphabet->alphaSize) >= 0) {
		return determinePower2Alpha_Buckets_Sarray(mappedString, d, suffixArrayPtr);
	}
	else {
		return determineAll_Buckets_Sarray(mappedString, d, suffixArrayPtr);
	}
}



/**
 * Constructs all buckets w.r.t. q-gram size d,
 * the up to prefix d sorted suffix array, and
 * the bucket-pointer table
 * Call determine_Buckets_Sarray_Sptrmap(...) instead
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @param sufPtrMapPtr   - ptr that will be filled with the bucket-pointer table
 * @return buckets containing pointers into the suffix array
 * @see determine_Buckets_Sarray_Sptrmap
 */
static Kbs_Ulong **determineAll_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr,
																					Kbs_Ulong ***sufPtrMapPtr) {

	register Kbs_Ulong **const buckets = determineAll_Buckets_Sarray(mappedString, d, suffixArrayPtr);
	register Kbs_Ulong strLen = mappedString->strLength;
	register Kbs_Ulong **const sufPtrMap = (Kbs_Ulong **)malloc(sizeof(Kbs_Ulong *) * (strLen+2*d+1));
	if (sufPtrMap == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}

	/* computation of first hashvalue */
	register const Kbs_Ulong alphabetSize = mappedString->alphabet->alphaSize;
	register Kbs_Uchar *mappedUcharArray = mappedString->str;
	register Kbs_Ulong tempPower = 1;
	register Kbs_Ulong hashCode = 0;
	register Kbs_Long i;
	for (i=d-1; i>=0; i--) {
		hashCode += mappedUcharArray[i] * tempPower;
		tempPower *= alphabetSize;
	}
	register const Kbs_Ulong tempModulo = kbs_power_Ulong(alphabetSize, d-1);
	mappedUcharArray += d;
	register Kbs_Ulong j;
	for (j=0; j<strLen-d; j++) {
		sufPtrMap[j] = (buckets[hashCode+1]) - 1;
		hashCode = hashCode % tempModulo;
		hashCode *= alphabetSize;
		hashCode += *mappedUcharArray;
		mappedUcharArray++;
	}
	const Kbs_Ulong firstHashCode = hashCode;
	for (j = strLen-d; j<strLen; j++) {
		sufPtrMap[*(buckets[hashCode])] = buckets[hashCode];
		(buckets[hashCode])++;
		hashCode = hashCode % tempModulo;
		hashCode *= alphabetSize;
		hashCode += *mappedUcharArray;
		mappedUcharArray++;
	}
	hashCode = firstHashCode;
	mappedUcharArray -= d;
	for (j = strLen-d; j<strLen; j++) {
		buckets[hashCode]--;
		hashCode = hashCode % tempModulo;
		hashCode *= alphabetSize;
		hashCode += *mappedUcharArray;
		mappedUcharArray++;
	}
	/* set the values in sufPtrMap[strLen..strLen+2*d] to [-1, -2, ..., -2*d]*/
	Kbs_Ulong *beginPtr = *suffixArrayPtr - 1;
	for (j=strLen; j<=strLen+2*d; j++) {
		sufPtrMap[j] = beginPtr--;
	}
	//*suffixArrayPtr = suffixArray;
	*sufPtrMapPtr = sufPtrMap;
	return buckets;
}


/**
 * Constructs all buckets w.r.t. q-gram size d,
 * the up to prefix d sorted suffix array, and
 * the bucket-pointer table
 * Call determine_Buckets_Sarray_Sptrmap(...) instead
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @param sufPtrMapPtr   - ptr that will be filled with the bucket-pointer table
 * @return buckets containing pointers into the suffix array
 * @see determine_Buckets_Sarray_Sptrmap
 */
static Kbs_Ulong **determinePower2Alpha_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr,
																					Kbs_Ulong ***sufPtrMapPtr) {

	register Kbs_Ulong strLen = mappedString->strLength;
	const Kbs_Int exp2 = (Kbs_Int) kbs_getExp_Ulong(2, mappedString->alphabet->alphaSize);
	if (exp2 < 0) {
		KBS_ERROR(KBS_ERROR_VALUEOUTOFBOUNDS);
		return NULL;
	}

	/* initialization of bucket table */
	register Kbs_Ulong **const buckets = determinePower2Alpha_Buckets_Sarray(mappedString, d, suffixArrayPtr);
	register Kbs_Ulong **const sufPtrMap = (Kbs_Ulong **)malloc(sizeof(Kbs_Ulong *) * (strLen+2*d+1));
	//sufPtrMap = (Kbs_Ulong **)malloc(sizeof(Kbs_Ulong *) * (strLen+1));
	if (sufPtrMap == NULL) {
		KBS_ERROR(KBS_ERROR_MALLOC);
	}

	/* computation of first hashvalue */
	register Kbs_Uchar *mappedUcharArray = mappedString->str;
	register Kbs_Ulong hashCode = 0;
	register Kbs_Ulong j;
	for (j=0; j<d; j++) {
		hashCode = hashCode << exp2;
		hashCode += mappedUcharArray[j];
	}
	register Kbs_Ulong tempModulo = (Kbs_Ulong)0;
	tempModulo = ~tempModulo;
	tempModulo = tempModulo << (exp2*(d-1));
	tempModulo = ~tempModulo;
	mappedUcharArray += d;
	for (j=0; j<strLen; j++) {
		sufPtrMap[j] = (buckets[hashCode+1]) - 1;
		hashCode = hashCode & tempModulo;
		hashCode = hashCode << exp2;
		hashCode = hashCode | (Kbs_Ulong)(*mappedUcharArray);
		mappedUcharArray++;
	}
	Kbs_Ulong *beginPtr = *suffixArrayPtr - 1;
	for (j=strLen; j<=strLen+2*d; j++) {
		sufPtrMap[j] = beginPtr--;
	}
	*sufPtrMapPtr = sufPtrMap;
	return buckets;
}



/**
 * Constructs all buckets w.r.t. q-gram size d,
 * the up to prefix d sorted suffix array, and
 * the bucket-pointer table
 * @param mappedString   - String of characters 0..alphabetsize-1
 * @param d              - size of q-gram
 * @param suffixArrayPtr - ptr that will be filled with the suffix array
 * @param sufPtrMapPtr   - ptr that will be filled with the bucket-pointer table
 * @return buckets containing pointers into the suffix array
 */
static Kbs_Ulong **determine_Buckets_Sarray_Sptrmap(const Kbs_Ustring *const mappedString,
																					register const Kbs_Ulong d,
																					Kbs_Ulong **suffixArrayPtr,
																					Kbs_Ulong ***sufPtrMapPtr) {
	if (kbs_getExp_Ulong(2, mappedString->alphabet->alphaSize) >= 0) {
		return determinePower2Alpha_Buckets_Sarray_Sptrmap(mappedString, d, suffixArrayPtr, sufPtrMapPtr);
	}
	else {
		return determineAll_Buckets_Sarray_Sptrmap(mappedString, d, suffixArrayPtr, sufPtrMapPtr);
	}
}






/*--------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------------------*/
/**
 * Sorts the alphabet concerning some weight concerning cc bucket size and alphabet frequency
 * Only works for mapped string with alphabet [0,alphaSize]
 * @param alphabet - the alphabet of the string
 * @param buckets - the bucket table
 * @param bucketsInLevel2Bucket - number of subbuckets of level-2 buckets
 * @param weightOnCC - the weight on buckets with same first and second character
 * @return the order of the alphabet according to the weight on buckets with same first and second character
 */
static Kbs_Uchar *getCharWeightedOrder_Alphabet(const Kbs_Alphabet *const alphabet,
																											const Kbs_Ulong **const buckets,
																											const Kbs_Ulong bucketsInLevel2Bucket,
																											const Kbs_Ulong weightOnCC){

	register const Kbs_Ulong alphabetSize = alphabet->alphaSize;
	Kbs_Ulong charWeight[alphabetSize];
	register const Kbs_Ulong tmpBucketFactor = bucketsInLevel2Bucket * (alphabetSize + 1);
	register Kbs_Ulong i;
	for (i=0; i<alphabetSize; i++) {
		charWeight[i] = (Kbs_Ulong)alphabet->charFreq[i];
		charWeight[i] -= weightOnCC * (buckets[i * tmpBucketFactor + bucketsInLevel2Bucket ]-buckets[i * tmpBucketFactor]);
	}

	register Kbs_Uchar *targetCharArray = (Kbs_Uchar *) malloc((alphabetSize + 1) * sizeof(Kbs_Uchar));
	for (i=0; i < alphabetSize; i++) {
		targetCharArray[i] = (Kbs_Uchar)i;
	}
	for (i=1; i < alphabet->alphaSize; i++) {
		register const Kbs_Ulong tmpWeight = charWeight[i];
		register Kbs_Long j = i;
		while(j>0 && tmpWeight < charWeight[targetCharArray[j-1]])  {
			targetCharArray[j] = targetCharArray[j-1];
			j--;
		}
		targetCharArray[j] = (Kbs_Uchar)i;
	}
	return targetCharArray;
}


/*---------------------------------------------------------------------------------------*/
Kbs_SuffixArray *kbs_buildDstepUsePrePlusCopyFreqOrder_SuffixArray(Kbs_Ustring *const seq, register const Kbs_Ulong d) {

	if (seq == NULL) {
		KBS_ERROR(KBS_ERROR_NULLPOINTER);
		return NULL;
	}
	if (seq->alphabet == NULL) {
		KBS_ERROR(KBS_ERROR_NULLPOINTER);
		return NULL;
	}
	register const Kbs_Ulong strLen = seq->strLength;
	if (d > strLen){
		printf("q-gram size higher than string length: %lu > %lu\n", d, strLen);
		KBS_ERROR(KBS_ERROR_VALUEOUTOFBOUNDS);
		return NULL;
	}
	if ( d > determineMaxD(seq->alphabet->alphaSize) ){
		printf("q-gram size = %lu, results in to many buckets\n", d);
		KBS_ERROR(KBS_ERROR_VALUEOUTOFBOUNDS);
		return NULL;
	}

	kbs_sort_Alphabet(seq->alphabet);
	/* computation of bucket table */
	kbs_convert_Ustring2RangeExtend(seq, d+d);

	Kbs_Ulong *suffixArray;
	Kbs_Ulong **sufPtrMap;
	const Kbs_Ulong **const buckets = determine_Buckets_Sarray_Sptrmap(seq, d, &suffixArray, &sufPtrMap);
	if (buckets == NULL) {
		KBS_ERROR(KBS_ERROR_NULLPOINTER);
		return NULL;
	}

	/* Sorting of all buckets */
	register const Kbs_Uchar *const mappedCharPtr = seq->str;
	const Kbs_Ulong alphabetSize = seq->alphabet->alphaSize;
	const Kbs_Ulong bucketsInLevel2Bucket = kbs_power_Ulong(alphabetSize, d-2);
	const Kbs_Ulong bucketsInLevel1Bucket = bucketsInLevel2Bucket * alphabetSize;
	const Kbs_Uchar *const alphaOrder = getCharWeightedOrder_Alphabet(seq->alphabet, buckets, bucketsInLevel2Bucket, 1);
	Kbs_Uchar *const isNotSortedLevel1Char = (Kbs_Uchar *) malloc(sizeof(Kbs_Uchar) * alphabetSize);
	memset(isNotSortedLevel1Char, '\1', sizeof(Kbs_Uchar) * alphabetSize);

	/* Sort all level-1 buckets */
	Kbs_Ulong *leftPtrList[alphabetSize];		/* just for the copy phase maintains all pointers to the right of the small buckets */
	Kbs_Ulong *rightPtrList[alphabetSize];		/* just for the copy phase maintains all pointers to the right of the small buckets */
	Kbs_Ulong i;
	for (i=0; i < alphabetSize; i++) {
		register const Kbs_Ulong c1 = alphaOrder[i];
		/* sort buckets cd to cz */
		register Kbs_Ulong j;
		for (j=i+1; j < alphabetSize; j++) {
			register const Kbs_Ulong c2 = alphaOrder[j];
			register const Kbs_Ulong tmpUlong = c1 * bucketsInLevel1Bucket + c2 * bucketsInLevel2Bucket;
			register Kbs_Ulong k;
			for (k = tmpUlong; k < tmpUlong + bucketsInLevel2Bucket; k++) {
				register Kbs_Ulong *const leftPtr = buckets[k];
				register Kbs_Ulong *const rightPtr = buckets[k+1] - 1;
				register Kbs_Long bucketSize;
				if ((bucketSize = rightPtr - leftPtr) > 0) {
					if (bucketSize < KBS_INSSORT_THRES_LEN)
						insSortViaPtr_SaBucket(leftPtr, rightPtr, d, d, sufPtrMap);
					else
						qSortInsSortViaPtr_SaBucket(leftPtr, rightPtr, d, d, sufPtrMap);
				}
			}
		}

		/* copy left buckets of cx */
		for (j=0; j<alphabetSize; j++) {
			leftPtrList[j] = buckets[j * bucketsInLevel1Bucket + c1 * bucketsInLevel2Bucket];
		}
		register Kbs_Ulong *leftPtr = buckets[c1 * bucketsInLevel1Bucket];
		while(leftPtr < leftPtrList[c1]) {
			register Kbs_Ulong tmpUlong;
			register Kbs_Uchar tmpUchar;
			if ( (tmpUlong = *leftPtr) && isNotSortedLevel1Char[tmpUchar = mappedCharPtr[--tmpUlong]]) {
				register Kbs_Ulong *tmpUlongPtr;
				sufPtrMap[tmpUlong] = tmpUlongPtr = leftPtrList[tmpUchar];
				leftPtrList[tmpUchar] = tmpUlongPtr + 1;
				*tmpUlongPtr = tmpUlong;
			}
			leftPtr++;
		}

		/* copy right buckets of cx */
		for (j=0; j<alphabetSize; j++) {
			rightPtrList[j] = buckets[j * bucketsInLevel1Bucket + (c1+1) * bucketsInLevel2Bucket];
		}
		register Kbs_Ulong *rightPtr = buckets[(c1+1)*bucketsInLevel1Bucket];
		while(leftPtr < rightPtr) {
			rightPtr--;
			register Kbs_Ulong tmpUlong;
			register Kbs_Uchar tmpUchar;
			if ((tmpUlong = *rightPtr) && isNotSortedLevel1Char[tmpUchar = mappedCharPtr[--tmpUlong]]) {
				register Kbs_Ulong *tmpUlongPtr;
				sufPtrMap[tmpUlong] = rightPtrList[tmpUchar] = tmpUlongPtr = rightPtrList[tmpUchar] - 1;
				*tmpUlongPtr = tmpUlong;
			}
		}
		isNotSortedLevel1Char[c1] = 0;
	}
/* End Sorting of all buckets */
	free(alphaOrder);
	free(isNotSortedLevel1Char);
	free(buckets);
	free(sufPtrMap);
 	Kbs_SuffixArray *const outputSA = (Kbs_SuffixArray *)malloc(sizeof(Kbs_SuffixArray));
 	outputSA->str = seq;
 	outputSA->posArray = suffixArray;
	outputSA->invPosArray = NULL;
 	outputSA->bucketIndices = NULL;
	outputSA->qGramSize = d;
	return outputSA;
}


