
#ifndef KBS_SUFFIXARRAYCONSTDSTEPANDPRENUM_H
#define KBS_SUFFIXARRAYCONSTDSTEPANDPRE_H

#include "kbs_String.h"
#include "kbs_SuffixArray.h"
#include "kbs_Time.h"
#include "kbs_Types.h"




/**
 * build the suffix array with the d-step algorithm
 * the bucket references are realized via pointers into suffix array
 * the bucket-pointers are kept during refinement
 * for each suffix its final position or the last position in its bucket is
 * hold in a separate array
 */
Kbs_SuffixArray *kbs_buildDstepUsePreNum_SuffixArray(Kbs_Ustring *const seq, register const Kbs_Ulong d);

/**
 * build the suffix array with the d-step algorithm
 * the bucket references are realized via pointers into suffix array
 * the bucket-pointers are not kept during refinement
 * for each suffix its final position or the last position in its bucket is
 * stored in an separate array
 * buckets are determined via the back-references in this separate array
 */
Kbs_SuffixArray *kbs_buildDstepUsePreNum_SuffixArray2(Kbs_Ustring *const seq, register const Kbs_Ulong d);

/**
 * build the suffix array with the d-step algorithm
 * the bucket references are realized via pointers into suffix array
 * the bucket-pointers are not kept during refinement
 * for each suffix its final position or the last position in its bucket is
 * stored in an separate array
 * the bucket array is freed before the computation back-reference array
 * buckets are determined via the back-references in this separate array
 */
Kbs_SuffixArray *kbs_buildDstepUsePreNum_SuffixArray3(Kbs_Ustring *const seq, register const Kbs_Ulong d);


/**
 * build the suffix array with the d-step algorithm
 * the bucket references are realized via pointers into suffix array
 * the bucket-pointers are kept during refinement
 * for each suffix its final position or the last position in its bucket is
 * hold in an separate array
 * For already sorted 1-level array a the 2-level buckets ya are determined
 * by scan of bucket a
 */
Kbs_SuffixArray *kbs_buildDstepUsePrePlusCopyNum_SuffixArray(Kbs_Ustring *const seq, register const Kbs_Ulong d);

/**
 * Build the suffix array with the bpr algorithm.
 * The bucket references are realized via indexes refering into the suffix array.
 * For each suffix its final position or the last position in its bucket is
 * hold in an separate array.
 * First the smallest level-1 buckets are determined
 * For the already sorted level-1 bucket 'a', the 2-level buckets 'ya' are determined
 * by a left-to-right scan of bucket 'a'.
 * The space demands are up to 10*(seq->strLength)
 * @param seq - string to build suffix array for
 * @param d - prefixlength for which the initial bucket sort is performed
 * @return the suffix array for the string seq with invPosArray and bucketIndices
 * @see kbs_buildDstepUsePrePlusCopyFreqOrder_SuffixArray(Kbs_Ustring *const seq, register const Kbs_Ulong d);
 */
Kbs_SuffixArray *kbs_buildDstepUsePrePlusCopyFreqOrderNum_SuffixArray(Kbs_Ustring *const seq, register const Kbs_Ulong d);



#endif
